@@ -17,10 +17,10 @@ sub new {
 sub iterator {
     my $self = shift;
 
-    my $position = $self->{_position} + 1;
+    my $position = $self->{_position};
     if ( $self->{_use_cache}
       && ( my $row_cache = $self->{_rows_cache}->[$position] ) ) {
-        $self->{_position} = $position;
+        $self->{_position} = $position + 1;
         return $row_cache;
     }
 
@@ -41,23 +41,27 @@ sub iterator {
         return;
     }
 
-    return $row if Scalar::Util::blessed($row);
+    my $obj;
+    if ( Scalar::Util::blessed($row) ) {
+        $obj = $row;
+    }
+    else {
+        $obj = $self->{row_class}->new(
+            {
+                row_data       => $row,
+                skinny         => $self->{skinny},
+                opt_table_info => $self->{opt_table_info},
+            }
+        );
 
-    my $obj = $self->{row_class}->new(
-        {
-            row_data       => $row,
-            skinny         => $self->{skinny},
-            opt_table_info => $self->{opt_table_info},
+        unless ($self->{_setup}) {
+            $obj->setup;
+            $self->{_setup}=1;
         }
-    );
-
-    unless ($self->{_setup}) {
-        $obj->setup;
-        $self->{_setup}=1;
     }
 
     $self->{_rows_cache}->[$position] = $obj if $self->{_use_cache};
-    $self->{_position} = $position;
+    $self->{_position} = $position + 1;
 
     return $obj;
 }
